@@ -15,17 +15,24 @@
  */
 package net.morilib.lisp;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2011/04/09
+ */
 /*package*/ final class ClosureClass extends Datum {
-	
+
+	//
 	private Datum params;
 	private CompiledCode body;
-	
-	
+
+	//
 	/*package*/ ClosureClass() {
 		// do nothing
 	}
-	
-	
+
+	//
 	/*package*/ ClosureClass(Datum params, CompiledCode body) {
 		if(params == null || body == null) {
 			throw new NullPointerException();
@@ -33,18 +40,17 @@ package net.morilib.lisp;
 		this.params   = params;
 		this.body     = body;
 	}
-	
-	
+
+	//
 	/*package*/ Datum getParameterList() {
 		return params;
 	}
-	
-	
+
+	//
 	/*package*/ CompiledCode getCode() {
 		return body;
 	}
-	
-	
+
 	/**
 	 * @param params the params to set
 	 */
@@ -58,10 +64,20 @@ package net.morilib.lisp;
 	/*package*/ void setCode(CompiledCode body) {
 		this.body = body;
 	}
-	
-	
+
+	//
 	public boolean isTypeProcedure() {
 		return true;
 	}
-	
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.Datum#toDisplayString(java.lang.StringBuilder)
+	 */
+	@Override
+	public void toDisplayString(StringBuilder buf) {
+		buf.append("#<closureClass" +
+				Integer.toString(System.identityHashCode(this), 16) +
+				">");
+	}
+
 }
