@@ -21,77 +21,87 @@ import java.util.List;
 import java.util.Map;
 
 import net.morilib.lisp.CompiledCode.Builder;
+import net.morilib.lisp.subr.BinaryArgs;
+import net.morilib.lisp.subr.UnaryArgs;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2011/05/24
+ */
 /*package*/ class CompilerImpl extends LispCompiler {
-	
+
+	//
 	//private static Logger _log = LogEnv.init("schlush.compiler");
-	
+
+	//
 	private LispMessage message;
-	
-	
+
+	//
 	/*package*/ CompilerImpl(LispMessage msg) {
 		message = msg;
 	}
-	
-	
+
+	//
 	/*package*/ Datum expandSyntax(
 			UserSyntax syn, Datum body,
-			Environment env2) {
+			Environment env2, boolean syncase) {
 		List<Datum> pat = syn.getPatternList();
 		List<Datum> tmp = syn.getTemplateList();
 		PatternDepthMap mp;
 		boolean mtc = false;
 		int ind = 0;
-		
+
 		//System.out.println("bb:" + body);
 		//System.out.println("pt:" + pat);
 		do {
-			// ѥȹ̤
+			// パターン照合結果を初期化
 			mp = new PatternDepthMap(syn.getParamList().get(ind));
-			
-			// ѥޥå
+
+			// パターンマッチング
 			mtc = PatternMatch.match(
 					pat.get(ind), body, mp, syn.getReservedSet());
 		} while(!mtc && ++ind < pat.size());
-		
+
 		if(mtc) {
 			try {
-				// ѥ˥ޥå
+				// パターンにマッチした
 				Environment menv = new Environment();
 				Datum t;
 				Map<Symbol, Symbol> box = new HashMap<Symbol, Symbol>();
-				
-				//// ѿ֤
+
+				//// ローカル変数を置き換える
 				//t = replaceLocalVals(tmp.get(ind), env, menv);
 				t = tmp.get(ind);
-				
-				// ƥץ졼ȥѥ
+
+				// テンプレートコンパイル
 				Datum tpl = PatternMatch.compileTemplate(t, mp);
 				//_log.finer(LispUtils.print(tpl));
-				
-				// ƥץ졼Ÿ
-				// ܥˡ֥ޥͳפΥޡɲä
+
+				// テンプレート展開
+				// シンボルに「マクロ引数由来」のマークを追加する
 				Datum res = PatternMatch.expand(
-						tpl, mp, syn, box, env2);
+						tpl, mp, syn, box, env2, syncase);
 				//_log.finer(LispUtils.print(res));
-				
-				// ѿ֤
+
+				// ローカル変数を置き換える
 				res = replaceLocalVals(res, env2, menv, true, 0);
 				//_log.finer(LispUtils.print(res));
-				
-				// ׾ɲä
+
+				// スコープ情報を追加する
 				res = PatternMatch.appendScope(res, mp, syn);
 				//_log.finer(LispUtils.print(res));
-				
-				// ޡ򳰤
+
+				// マークを外す
 				res = PatternMatch.markReplace(box, res, false);
-				
+
 				// unwrap
 				res = PatternMatch.gtUnwrap(res);
-				
+
 				//_log.finer(LispUtils.print(res));
-				//System.out.println("ex:" + LispUtils.print(res));
-				
+//				System.out.println("ex:" + LispUtils.print(res));
+
 				return res;
 			} catch(PatternDepthException e) {
 				e.printStackTrace();
@@ -103,7 +113,8 @@ import net.morilib.lisp.CompiledCode.Builder;
 			//throw new LispException("malformed " + syn.getName());
 		}
 	}
-	
+
+	//
 	private Datum expandLambda(
 			UserSyntax syn,
 			Datum body,
@@ -114,7 +125,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 		CompiledCode.Builder mbuild = new CompiledCode.Builder();
 		ClosureClass clm = new ClosureClass(
 				Nil.NIL, syn.getLambda());
-		
+
 		mbuild.addPush(clm);
 		mbuild.addBeginList();
 		mbuild.addEndList();
@@ -126,16 +137,17 @@ import net.morilib.lisp.CompiledCode.Builder;
 		mbuild.addEndList();
 		mbuild.addCall();
 		mbuild.addReturnOp();
-		
+
 		// execute lambda
 		Datum res = exec.exec(mbuild.getCodeRef(), env2, memento);
 		//if(res == Nil.NIL) {
 		//	throw message.getError("err.syntaxcase.empty");
 		//}
-		//System.out.println("rs:" + res);
+//		System.out.println("rs:" + res);
 		return res;
 	}
-	
+
+	//
 	private void compileArgsBind(
 			Datum bcdr,
 			Environment env,
@@ -147,7 +159,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 			LispCompiler.MiscInfo syncased) {
 		Datum prms = symcall.getCdr();
 		List<Datum> vals = new ArrayList<Datum>();
-		
+
 		while(true) {
 			if(prms == Nil.NIL) {
 				if(bcdr != Nil.NIL) {
@@ -166,12 +178,12 @@ import net.morilib.lisp.CompiledCode.Builder;
 			} else if(prms instanceof Cons) {
 				Datum a1 = ((Cons)prms).getCar();
 				Datum a2 = ((Cons)bcdr).getCar();
-				
+
 				compile(a2, env, builder, symcall, false, symlist,
 						exec, memento, syncased);
 				//builder.addBind(a1);
 				vals.add(a1);
-				
+
 				prms = ((Cons)prms).getCdr();
 				bcdr = ((Cons)bcdr).getCdr();
 			} else {
@@ -179,15 +191,16 @@ import net.morilib.lisp.CompiledCode.Builder;
 				//throw new LispException("Invalid type");
 			}
 		}
-		
+
 		for(int i = vals.size() - 1; i >= 0; i--) {
 			builder.addBind(vals.get(i));
 		}
 	}
-	
+
+	//
 	/*private int poppos(Datum bcar, List<Cons> symlist) {
 		int res = 0;
-		
+
 		for(Cons c : symlist) {
 			if(bcar.equals(c.getCar())) {
 				return res;
@@ -197,7 +210,8 @@ import net.morilib.lisp.CompiledCode.Builder;
 		}
 		return -1;
 	}*/
-	
+
+	//
 	private void compileSexp(
 			Datum bcar,
 			Datum bcdr,
@@ -210,7 +224,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 			IntStack memento,
 			LispCompiler.MiscInfo syncased) {
 		//int pos = -1;
-		
+
 		if(istail && bcar.equals(symcall.getCar())) {
 			compileArgsBind(
 					bcdr, env, builder, symcall, symlist,
@@ -219,7 +233,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 		/*} else if(istail && (pos = poppos(bcar, symlist)) >= 0) {
 			Cons       s2 = symlist.get(pos);
 			List<Cons> sl = symlist.subList(pos + 1, symlist.size());
-			
+
 			builder.addRewindEnv(pos);
 			compileArgsBind(bcdr, env, builder, s2, sl);
 			builder.addRewindCont(pos);*/
@@ -229,7 +243,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 			compileArgs(
 					bcdr, env, builder, symcall, symlist,
 					exec, memento, syncased);
-			
+
 			if(istail) {
 				builder.addCallTail(symlist.size());
 			} else {
@@ -237,8 +251,10 @@ import net.morilib.lisp.CompiledCode.Builder;
 			}
 		}
 	}
-	
-	
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispCompiler#compileArgs(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.CompiledCode.Builder, net.morilib.lisp.Cons, java.util.List, net.morilib.lisp.CodeExecutor, net.morilib.lisp.IntStack, net.morilib.lisp.LispCompiler.MiscInfo)
+	 */
 	public void compileArgs(
 			Datum bcdr,
 			Environment env,
@@ -262,7 +278,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 						symcall, false, symlist,
 						exec, memento, syncased);
 				builder.addAppendList();
-				
+
 				bcdr = ((Cons)bcdr).getCdr();
 			} else {
 				throw message.getError("err.type.invalid");
@@ -270,29 +286,31 @@ import net.morilib.lisp.CompiledCode.Builder;
 			}
 		}
 	}
-	
+
+	//
 	private Datum getSym(Datum bcar, Environment env) {
 		Datum ddd;
-		
+
 		if(bcar instanceof Symbol) {
 			ddd = env.findDatum((Symbol)bcar);
 		} else {
 			SymbolScope ss   = (SymbolScope)bcar;
 			Environment menv =
 				ss.getUserSyntax().getCompileEnv();
-			
+
 			//System.out.println(ss);
 			//System.out.println(menv);
 			ddd = menv.findDatum(ss.getSymbol());
 		}
 		return ddd;
 	}
-	
+
+	//
 	private UserSyntax getUSyn(Datum d4, Environment env) {
 		if(!(d4 instanceof SymbolName)) {
 			return null;
 		}
-		
+
 		Datum d5 = getSym(d4, env);
 		if(d5 instanceof UserSyntax) {
 			return (UserSyntax)d5;
@@ -300,7 +318,10 @@ import net.morilib.lisp.CompiledCode.Builder;
 			return null;
 		}
 	}
-	
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispCompiler#compile(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.CompiledCode.Builder, boolean, net.morilib.lisp.Cons, boolean, java.util.List, net.morilib.lisp.CodeExecutor, net.morilib.lisp.IntStack, net.morilib.lisp.LispCompiler.MiscInfo)
+	 */
 	public void compile(
 			Datum body,          // S-expression to be compiled
 			Environment env,     // environment to be compiled
@@ -314,23 +335,23 @@ import net.morilib.lisp.CompiledCode.Builder;
 			MiscInfo syncased) {
 		if(body instanceof SymbolName) {
 			Datum ddd = getSym(body, env);
-			
+
 			if(ddd instanceof UserSyntax) {
 				UserSyntax sn = (UserSyntax)ddd;
 				Datum exp;
 				LispCompiler.MiscInfo mif = syncased;
-				
+
 				if(sn.getLambda() == null) {
 					builder.addReferSymbol(body);
 				} else {
 					Datum e2 = expandLambda(
 							sn, body, env, exec, memento);
-					
+
 					exp = e2;
 					if(mif.getUserSyntax() == null) {
 						mif = new LispCompiler.MiscInfo(sn);
 					}
-					
+
 					compile(exp, env, builder, toplevel,
 							symcall, istail, symlist,
 							exec, memento, mif);
@@ -343,19 +364,19 @@ import net.morilib.lisp.CompiledCode.Builder;
 		} else if(body instanceof Cons) {
 			Datum bcar = ((Cons)body).getCar();
 			Datum bcdr = ((Cons)body).getCdr();
-			
+
 			if(bcar instanceof Symbol || bcar instanceof SymbolScope) {
 				Datum ddd = getSym(bcar, env);
-				
+
 				if(ddd instanceof SynSetS) {
 					if(bcdr instanceof Cons) {
 						Datum d4 = ((Cons)bcdr).getCar();
 						UserSyntax sn;
-						
+
 						if((sn = getUSyn(d4, env)) == null) {
 							Syntax syn = (Syntax)ddd;
 							Datum dz = bcdr;
-							
+
 							syn.compile(
 									dz, env, this, builder, toplevel,
 									symcall, istail, message, symlist,
@@ -364,7 +385,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 							//Environment env2 = sn.getCompileEnv();
 							Datum exp;
 							LispCompiler.MiscInfo mif = syncased;
-							
+
 							if(sn.getLambda() == null) {
 								//exp = expandSyntax(sn, body, env2);
 								throw message.getError(
@@ -372,13 +393,13 @@ import net.morilib.lisp.CompiledCode.Builder;
 							} else {
 								Datum e2 = expandLambda(
 										sn, body, env, exec, memento);
-								
+
 								exp = e2;
 								if(mif.getUserSyntax() == null) {
 									mif = new LispCompiler.MiscInfo(sn);
 								}
 							}
-							
+
 							compile(exp, env, builder, toplevel,
 									symcall, istail, symlist,
 									exec, memento, mif);
@@ -390,7 +411,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 					// syntax
 					Syntax syn = (Syntax)ddd;
 					Datum dz = bcdr;
-					
+
 					/*if(ddd instanceof SynSyntax) {
 						//Cons r0 = new Cons();
 						//Cons r1 = new Cons();
@@ -400,14 +421,14 @@ import net.morilib.lisp.CompiledCode.Builder;
 								env, this,
 								new Environment(),
 								message, true, 1);
-						
+
 						//r0.setCar(bcar);
 						//r0.setCdr(r1);
 						//r1.setCar(r2);
 						//r2.setCar(dz);
 						//dz = r2;
 					}*/
-					
+
 					syn.compile(dz, env, this, builder, toplevel,
 							symcall, istail, message, symlist,
 							exec, memento, syncased);
@@ -417,13 +438,13 @@ import net.morilib.lisp.CompiledCode.Builder;
 					Environment env2 = usyn.getCompileEnv();
 					Datum exp;
 					LispCompiler.MiscInfo mif = syncased;
-					
+
 					if(usyn.getLambda() == null) {
-						exp = expandSyntax(usyn, body, env2);
+						exp = expandSyntax(usyn, body, env2, false);
 					} else {
 						Datum e2 = expandLambda(
 								usyn, body, env, exec, memento);
-						
+
 						//CompiledCode.Builder mbuild =
 						//	new CompiledCode.Builder();
 						//compile(e2, env, mbuild, toplevel,
@@ -436,15 +457,19 @@ import net.morilib.lisp.CompiledCode.Builder;
 						if(mif.getUserSyntax() == null) {
 							mif = new LispCompiler.MiscInfo(usyn);
 						}
+
+//						System.out.println("e2:" + exp);
+						exp = AUX_S.call("letren", exp);
+//						System.out.println("e3:" + exp);
 					}
-					
+
 					compile(exp, env, builder, toplevel,
 							symcall, istail, symlist,
 							exec, memento, mif);
 				} else if(ddd instanceof Macro) {
 					// traditional macro
 					Datum exp = expandMacro(body, env, exec, memento);
-					
+
 					compile(exp, env, builder, toplevel,
 							symcall, istail, symlist,
 							exec, memento, syncased);
@@ -461,7 +486,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 						env, this,
 						new Environment(),
 						message, true, 1);
-				
+
 				((Syntax)bcar).compile(
 						dz, env, this, builder, toplevel,
 						symcall, istail, message, symlist,
@@ -477,8 +502,8 @@ import net.morilib.lisp.CompiledCode.Builder;
 		}
 		//builder.addReturnOp();
 	}
-	
-	
+
+	//
 	private Datum replaceLocalValsSexp(
 			Datum bcar,
 			Datum bcdr,
@@ -486,15 +511,15 @@ import net.morilib.lisp.CompiledCode.Builder;
 			Environment ienv,
 			int ttype) {
 		Cons res = new Cons();
-		
+
 		res.setCar(replaceLocalVals(bcar, env, ienv, false, ttype));
-		
+
 		res.setCdr(replaceLocalValsArgs(bcdr, env, ienv, ttype));
-		
+
 		return res;
 	}
-	
-	
+
+	//
 	/*package*/ Datum replaceLocalValsOnly(
 			Datum body,
 			Environment env,
@@ -502,16 +527,16 @@ import net.morilib.lisp.CompiledCode.Builder;
 			boolean toplv,
 			int ttype) {
 		if(body instanceof SymbolName) {
-			// «ѿϤʤ
+			// 束縛変数はない
 			Symbol sym = ((SymbolName)body).getSymbol();
 			Datum d = ienv.findDatum(sym);
-			
+
 			return (d != null) ? d : body;
 		} else if(body instanceof Cons) {
 			Datum bcar = ((Cons)body).getCar();
 			Datum bcdr = ((Cons)body).getCdr();
 			Cons  res  = new Cons();
-			
+
 			res.setCar(replaceLocalValsOnly(
 					bcar, env, ienv, toplv, ttype));
 			res.setCdr(replaceLocalValsOnly(
@@ -521,7 +546,8 @@ import net.morilib.lisp.CompiledCode.Builder;
 			return body;
 		}
 	}
-	
+
+	//
 	private Datum replaceLocalValsWithSyn(
 			Datum bcdr,
 			Environment env,
@@ -531,72 +557,72 @@ import net.morilib.lisp.CompiledCode.Builder;
 		ConsIterator    itr = new ConsIterator(bcdr);
 		ConsListBuilder re2 = new ConsListBuilder();
 		ConsIterator    it2;
-		
+
 		if(!itr.hasNext()) {
 			throw message.getError("err.withsyntax.malform");
 		}
 		res.append(itr.next());
-		
+
 		if(!itr.hasNext()) {
 			throw message.getError("err.withsyntax.malform");
 		}
-		
+
 		it2 = new ConsIterator(itr.next());
 		while(it2.hasNext()) {
 			ConsListBuilder re3 = new ConsListBuilder();
 			ConsIterator    it3 = new ConsIterator(it2.next());
-			
+
 			if(!it3.hasNext()) {
 				throw message.getError("err.withsyntax.malform");
 			}
 			re3.append(it3.next());
-			
+
 			if(!it3.hasNext()) {
 				throw message.getError("err.withsyntax.malform");
 			}
 			re3.append(replaceLocalValsOnly(
 					it3.next(), env, ienv, false, ttype));
-			
+
 			if(it3.hasNext()) {
 				throw message.getError("err.withsyntax.malform");
 			}
-			
+
 			re2.append(re3.get());
 		}
 		res.append(re2.get());
-		
+
 		while(itr.hasNext()) {
 			res.append(replaceLocalValsOnly(
 					itr.next(), env, ienv, false, ttype));
 		}
-		
+
 //		System.out.println("wb:" + bcdr);
 //		System.out.println("ws:" + res.get());
 		return res.get();
 	}
-	
-	
+
+	//
 	public Datum replaceLocalValsArgs(
 			Datum bcdr,
 			Environment env,
 			Environment ienv,
 			int ttype) {
 		List<Datum> res = new ArrayList<Datum>();
-		
+
 		while(true) {
 			if(bcdr instanceof Nil) {
-				// «ѿϤʤ
+				// 束縛変数はない
 				return LispUtils.listToCons(res);
 				//break;
 			} else if(bcdr instanceof Atom) {
-				// «ѿϤʤ
+				// 束縛変数はない
 				return LispUtils.listToCons(res,
 						replaceLocalVals(bcdr, env, ienv, false, ttype));
 				//break;
 			} else if(bcdr instanceof Cons) {
 				res.add(replaceLocalVals(
 						((Cons)bcdr).getCar(), env, ienv, false, ttype));
-				
+
 				bcdr = ((Cons)bcdr).getCdr();
 			} else {
 				throw message.getError("err.type.invalid");
@@ -604,8 +630,8 @@ import net.morilib.lisp.CompiledCode.Builder;
 			}
 		}
 	}
-	
-	
+
+	//
 	public Datum replaceLocalVals(
 			Datum body,
 			Environment env,
@@ -613,13 +639,13 @@ import net.morilib.lisp.CompiledCode.Builder;
 			boolean toplv,
 			int ttype) {
 		if(body instanceof SymbolName) {
-			// «ѿϤʤ
+			// 束縛変数はない
 			Symbol sym = ((SymbolName)body).getSymbol();
 			Datum d = ienv.findDatum(sym);
-			
+
 			return (d != null) ? d : body;
 		//} else if(body instanceof SymbolScope) {
-		//	// «ѿϤʤ
+		//	// 束縛変数はない
 		//	Symbol sym = ((SymbolScope)body).getSymbol();
 		//	Datum d = ienv.findDatum(sym);
 		//	
@@ -627,29 +653,29 @@ import net.morilib.lisp.CompiledCode.Builder;
 		} else if(body instanceof Cons) {
 			Datum bcar = ((Cons)body).getCar();
 			Datum bcdr = ((Cons)body).getCdr();
-			
+
 			if(bcar instanceof Symbol || bcar instanceof SymbolScope) {
 				Symbol sym = ((SymbolName)bcar).getSymbol();
 				Datum  d2  = getSym(bcar, env);
-				
+
 				if(d2 instanceof SynSetS) {
 					if(bcdr instanceof Cons) {
 						Datum d4 = ((Cons)bcdr).getCar();
 						UserSyntax sn;
-						
+
 						if((sn = getUSyn(d4, env)) == null) {
 							Syntax syn = (Syntax)d2;
 							Cons res = new Cons();
-							
+
 							res.setCar(bcar);
 							res.setCdr(syn.replaceLocalVals(
 									bcdr, env, this, ienv,
 									message, toplv, ttype));
-							
+
 							return res;
 						} else {
 							Datum exp;
-							
+
 							if(sn.getLambda() == null) {
 								throw message.getError(
 										"err.set.malform");
@@ -664,12 +690,12 @@ import net.morilib.lisp.CompiledCode.Builder;
 				} else if(d2 instanceof Syntax) {
 					Syntax syn = (Syntax)d2;
 					Cons res = new Cons();
-					
+
 					res.setCar(bcar);
 					res.setCdr(syn.replaceLocalVals(
 							bcdr, env, this, ienv,
 							message, toplv, ttype));
-					
+
 					return res;
 				} else if(SynSyntax.WITH_SYNTAX.equals(sym)) {
 					return replaceLocalValsWithSyn(
@@ -679,7 +705,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 					// user defined syntax (RnRS macro)
 					UserSyntax usyn = (UserSyntax)d2;
 					Datum exp;
-					
+
 					if(usyn.getLambda() == null) {
 						exp = replaceLocalValsSexp(
 								bcar, bcdr, env, ienv, ttype);
@@ -696,31 +722,33 @@ import net.morilib.lisp.CompiledCode.Builder;
 						bcar, bcdr, env, ienv, ttype);
 			}
 		} else {
-			// «ѿϤʤ
+			// 束縛変数はない
 			return body;
 		}
 	}
-	
-	
-	// ͤѹե饰ξ뤿ΥǡǼ饹
+
+	//
+	// 戻り値と変更フラグを両方管理するためのデータ格納クラス
 	private class D2 {
-		private Datum d;         // ͤȤʤǡ
-		private boolean dirty;   // ѹ줿Ȥtrue
-		
+		private Datum d;         // 戻り値となるデータ
+		private boolean dirty;   // 変更されたときはtrue
+
 		private D2() {
 			// default
 		}
-		
+
 		private D2(Datum d, boolean dirty) {
 			this.d = d;
 			this.dirty = dirty;
 		}
-		
+
 	}
-	
+
+	//
 	/*private static final Symbol DEFMACRO = 
 		Symbol.getSymbol("define-macro");*/
-	
+
+	//
 	private D2 expandMacroSexp(
 			Datum car,
 			Datum cdr,
@@ -729,17 +757,17 @@ import net.morilib.lisp.CompiledCode.Builder;
 			IntStack memento) {
 		Cons res = new Cons();
 		boolean dirty;
-		
+
 		D2 d2 = expandMacro1i(car, env, exec, memento);
 		res.setCar(d2.d);
 		dirty = d2.dirty;
-		
+
 		List<Datum> lst = new ArrayList<Datum>();
 		while(true) {
 			if(cdr instanceof Cons) {
 				Cons c = (Cons)cdr;
 				D2 d3 = expandMacro1i(c.getCar(), env, exec, memento);
-				
+
 				lst.add(d3.d);
 				dirty = d3.dirty || dirty;
 				cdr = c.getCdr();
@@ -752,22 +780,22 @@ import net.morilib.lisp.CompiledCode.Builder;
 			}
 		}
 	}
-	
-	
+
+	//
 	private D2 expandMacro1i(
 			Datum body,
 			Environment env,
 			CodeExecutor exec,
 			IntStack memento) {
 		D2 res = new D2();
-		
+
 		if(body instanceof Cons) {
 			Cons c = (Cons)body;
-			
+
 			if(c.getCar() instanceof SymbolName) {
 				Datum d = env.findDatum(
 						((SymbolName)c.getCar()).getSymbol());
-				
+
 				if(d instanceof MacroDefinition) {
 					return new D2(body, false);
 				} else if(d instanceof Macro) {
@@ -776,7 +804,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 					Closure cl = m.getClosure();
 					Environment nenv = new Environment(env);
 					//Object memento = exec.newMemento();
-					
+
 					IntLispUtils.bindLocal(
 							cl.getParameterList(),
 							c.getCdr(), nenv, message);
@@ -804,8 +832,10 @@ import net.morilib.lisp.CompiledCode.Builder;
 			return res;
 		}
 	}
-	
-	
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispCompiler#expandMacro1(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.CodeExecutor, net.morilib.lisp.IntStack)
+	 */
 	public Datum expandMacro1(
 			Datum body,
 			Environment env,
@@ -813,23 +843,25 @@ import net.morilib.lisp.CompiledCode.Builder;
 			IntStack memento) {
 		return expandMacro1i(body, env, exec, memento).d;
 	}
-	
-	
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.LispCompiler#expandMacro(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.CodeExecutor, net.morilib.lisp.IntStack)
+	 */
 	public Datum expandMacro(
 			Datum body,
 			Environment env,
 			CodeExecutor exec,
 			IntStack memento) {
 		D2 loop = new D2(body, false);
-		
-		// Ÿޥʤʤޤǥ롼
+
+		// 展開するマクロがなくなるまでループ
 		do {
 			Datum b = loop.d;
-			
-			// define-macroΤȤϥޥŸʤ
+
+			// define-macroのときはマクロ展開しない
 			if(b instanceof Cons) {
 				Datum c1 = ((Cons)b).getCar();
-				
+
 				if(c1 instanceof Symbol) {
 					if(env.findDatum(c1) instanceof MacroDefinition) {
 						return b;
@@ -837,7 +869,7 @@ import net.morilib.lisp.CompiledCode.Builder;
 				} else if(c1 instanceof SymbolScope) {
 					Datum d1 =
 						env.findDatum(((SymbolScope)c1).getSymbol());
-					
+
 					if(d1 instanceof SynDefineMacro) {
 						return b;
 					}
@@ -846,12 +878,180 @@ import net.morilib.lisp.CompiledCode.Builder;
 					return body;
 				}*/
 			}
-			
-			// ޥŸ
+
+			// マクロ展開
 			loop = expandMacro1i(b, env, exec, memento);
 		} while(loop.dirty);
-		
+
 		return loop.d;
 	}
-	
+
+	//
+	private static final Scheme AUX_S;
+
+	//
+	private static class EqWithSyn extends BinaryArgs {
+
+		@Override
+		protected Datum execute(
+				Datum c1a, Datum c2a,
+				Environment env, LispMessage mesg) {
+			if(c1a instanceof SymbolScope &&
+					c2a instanceof SymbolScope) {
+				SymbolScope s1 = (SymbolScope)c1a;
+				SymbolScope s2 = (SymbolScope)c2a;
+
+				return 	LispBoolean.getInstance(
+						s1.getSymbol().equals(s2.getSymbol()) &&
+						s1.isSameScope(s2));
+			}
+			return LispBoolean.FALSE;
+		}
+
+	}
+
+	//
+	private static class RmWithSyn extends UnaryArgs {
+
+		@Override
+		protected Datum execute(
+				Datum c1a,
+				Environment env, LispMessage mesg) {
+			if(c1a instanceof SymbolScope) {
+				SymbolScope s1 = (SymbolScope)c1a;
+
+				return s1.getSymbol();
+			}
+			return c1a;
+		}
+
+	}
+
+	static {
+		AUX_S = new Scheme(
+				Scheme.newRnRSEnv(Scheme.SCHEME_VERSION),
+				LispMessage.getInstance());
+
+		AUX_S.set("samescp?", new EqWithSyn());
+		AUX_S.set("remvscp", new RmWithSyn());
+		AUX_S.exec(
+				"(define (mapv f vec)" +
+				"  (let ((v1 (make-vector (vector-length vec))))" +
+				"    (let loop ((i (vector-length vec)))" +
+				"      (cond ((zero? i) v1)" +
+				"            (else" +
+				"              (vector-set!" +
+				"                v1" +
+				"                (- i 1)" +
+				"                (f (vector-ref vec (- i 1))))" +
+				"              (loop (- i 1)))))))");
+		AUX_S.exec(
+				"(define (asss x lis)" +
+//				"(display (samescp? (caar lis) x)) (newline)" +
+				"  (cond ((null? lis) #f)" +
+				"        ((not (pair? (car lis)))" +
+				"          (error (get-default-message" +
+				"            'err.require.pair)))" +
+				"        ((samescp? (caar lis) x) (car lis))" +
+				"        (else (asss x (cdr lis)))))");
+		AUX_S.exec("(define (letren x) (lren0 x '() #f))");
+		AUX_S.exec(
+				"(define (lren0 x as lv)" +
+//				"(display x) (newline)" +
+//				"(display as) (newline)" +
+				"  (cond ((and #;(not lv)" +
+				"              (pair? x)" +
+				"              (memq (remvscp (car x)) '(let letrec)))" +
+				"          (if (symbol? (cadr x))" +
+				"              (chletn x" +
+				"                      as" +
+				"                      (cons" +
+				"                        (cons (cadr x)" +
+				"                              (remvscp (cadr x)))" +
+				"                        (append" +
+				"                          (makeasc (caddr x))" +
+				"                          as))" +
+				"                      lv)" +
+				"              (chlet  x" +
+				"                      as" +
+				"                      (append (makeasc (cadr x)) as)" +
+				"                      lv)))" +
+				"        ((and #;(not lv)" +
+				"              (pair? x)" +
+				"              (memq (remvscp (car x)) '(let* letrec*)))" +
+				"          (chlet* x" +
+				"                  as" +
+				"                  (append (makeasc (cadr x)) as)" +
+				"                  lv))" +
+				"        ((and #;(not lv)" +
+				"              (pair? x)" +
+				"              (memq (remvscp (car x)) '(lambda)))" +
+				"          (let ((as2 (append (makeascl (cadr x)) as)))" +
+				"            (cons (car x)" +
+				"             (cons (lren0 (cadr x) as2 lv)" +
+				"              (lren0 (cddr x) as2 lv)))))" +
+				"        ((pair? x)" +
+				"          (cons (lren0 (car x) as lv)" +
+				"                (lren0 (cdr x) as lv)))" +
+				"        ((vector? x)" +
+				"          (mapv (lambda (x) (lren0 x as lv)) x))" +
+				"        ((and #;(not lv) (asss x as))" +
+				"          (cdr (asss x as)))" +
+				"        (else x)))" +
+				"");
+		AUX_S.exec(
+				"(define (makeasc x)" +
+				"  (cond ((null? x) '())" +
+				"        (else (cons (cons (caar x)" +
+				"                          (remvscp (caar x)))" +
+				"                    (makeasc (cdr x))))))");
+		AUX_S.exec(
+				"(define (makeascl x)" +
+				"  (cond ((null? x) '())" +
+				"        (else" +
+				"         (cons (cons (car x) (remvscp (car x)))" +
+				"          (makeascl (cdr x))))))");
+		AUX_S.exec(
+				"(define (chlet x a0 as lv)" +
+				"  (cons (car x)" +
+				"        (cons (map (lambda (x)" +
+				"                     (cons (asssif (car x) as)" +
+				"                           (lren0 (cdr x) a0 lv)))" +
+				"                   (cadr x))" +
+				"              (lren0 (cddr x) as lv))))");
+		AUX_S.exec(
+				"(define (chlet*-aux x a0 lv)" +
+				"  (cond ((null? x) '())" +
+				"        (else" +
+				"          (cons" +
+				"            (cons (remvscp (caar x))" +
+				"                  (lren0 (cdar x) a0 lv))" +
+				"            (chlet*-aux" +
+				"              (cdr x)" +
+				"              (cons" +
+				"                (cons (caar x) (remvscp (caar x)))" +
+				"                a0)" +
+				"              lv)))))");
+		AUX_S.exec(
+				"(define (chlet* x a0 as lv)" +
+				"  (cons (car x)" +
+				"        (cons (chlet*-aux (cadr x) a0 lv)" +
+				"              (lren0 (cddr x) as lv))))");
+		AUX_S.exec(
+				"(define (chletn x a0 as lv)" +
+//				"(display as) (newline)" +
+				"  (cons (car x) #;(asssif (car x) as)" +
+				"        (cons (asssif (cadr x) as)" +
+				"              (cons (map (lambda (x)" +
+				"                           (cons (asssif (car x) as)" +
+				"                                 (lren0" +
+				"                                   (cdr x) a0 lv)))" +
+				"                         (caddr x))" +
+				"                    (lren0 (cdddr x) as lv)))))");
+		AUX_S.exec(
+				"(define (asssif x as)" +
+//				"(display (asss x as)) (newline)" +
+				"  (if (asss x as) (cdr (asss x as)) x))");
+	}
+
 }
