@@ -24,7 +24,7 @@ public class IntegerToChar extends UnaryArgs {
 
 	@Override
 	protected Datum execute(Datum c1a, Environment env, LispMessage mesg) {
-		int i = SubrUtils.getSmallIntegerExact(c1a);
+		int i = SubrUtils.getSmallIntegerExact(c1a, mesg);
 		
 		if(i < 0) {
 			throw mesg.getError("err.require.int.nonnegative", c1a);
